// -*- C++ -*-
//
//  This file is part of Siena, a wide-area event notification system.
//  See http://www.inf.usi.ch/carzaniga/siena/
//
//  Authors: Antonio Carzaniga
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2001-2003 University of Colorado
//
//  Siena is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//  
//  Siena is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with Siena.  If not, see <http://www.gnu.org/licenses/>.
//
#ifndef SIMPLE_FWD_TYPES_H
#define SIMPLE_FWD_TYPES_H

#include <list>
#include <map>

#include <siena/types.h>

class simple_value: public siena::value {
public:
    virtual siena::type_id	type() const;
    virtual siena::int_t	int_value() const;
    virtual siena::string_t	string_value() const;
    virtual siena::bool_t	bool_value() const;
    virtual siena::double_t	double_value() const;

    simple_value();
    simple_value(siena::int_t x);
    simple_value(siena::bool_t x);
    simple_value(siena::double_t x);
    simple_value(const siena::string_t & x);

private:
    siena::type_id t;
    union {
	siena::int_t i;
	siena::bool_t b;
	siena::double_t d;
    };
    siena::string_t s;
};

typedef std::map<siena::string_t, const simple_value *> attribute_map;

class simple_attribute : public siena::message::iterator {
public:
    virtual siena::string_t	name() const;
    virtual siena::type_id	type() const;
    virtual siena::int_t	int_value() const;
    virtual siena::string_t	string_value() const;
    virtual siena::bool_t	bool_value() const;
    virtual siena::double_t	double_value() const;
    virtual bool		next();

    simple_attribute(attribute_map::const_iterator b, 
		     attribute_map::const_iterator e);
private:
    attribute_map::const_iterator i;
    attribute_map::const_iterator end;
};

class simple_message: public siena::message {
public:
    virtual iterator *	first() const;
    virtual iterator *	find(const siena::string_t & name) const;
    virtual bool	contains(const siena::string_t & name) const;
    bool		add(const siena::string_t & name, 
			    const simple_value * a);

    simple_message();

    virtual ~simple_message();

private:
    attribute_map attrs;
};

class simple_op_value: public simple_value {
public:
    virtual siena::operator_id	op() const;

    simple_op_value(siena::operator_id xo);
    simple_op_value(siena::operator_id xo, siena::int_t x);
    simple_op_value(siena::operator_id xo, siena::bool_t x);
    simple_op_value(siena::operator_id xo, siena::double_t x);
    simple_op_value(siena::operator_id xo, const siena::string_t & x);

private:
    siena::operator_id o;
};

typedef std::multimap<siena::string_t, const simple_op_value *> constraint_map;

class simple_constraint: public siena::filter::iterator {
public:
    virtual siena::string_t	name() const;
    virtual siena::type_id	type() const;
    virtual siena::int_t	int_value() const;
    virtual siena::string_t	string_value() const;
    virtual siena::bool_t	bool_value() const;
    virtual siena::double_t	double_value() const;
    virtual siena::operator_id	op() const;
    virtual bool		next();
    simple_constraint(constraint_map::const_iterator b, 
		      constraint_map::const_iterator e);
private:
    constraint_map::const_iterator i;
    constraint_map::const_iterator end;
};

class simple_filter: public siena::filter {
public:
    virtual iterator *		first() const;
    void			add(const siena::string_t name, 
				    const simple_op_value * v);

    simple_filter();
    virtual ~simple_filter();

private:
    constraint_map constraints;
};

typedef std::list<simple_filter *> filter_list;

class simple_predicate_i: public siena::predicate::iterator {
public:
    virtual filter::iterator * first() const;
    virtual bool		next();

    simple_predicate_i(filter_list::const_iterator b,
		       filter_list::const_iterator e);
private:
    filter_list::const_iterator i;
    filter_list::const_iterator end;
};

class simple_predicate : public siena::predicate {
public:
    virtual iterator *		first() const;
    void			add(simple_filter * v);
    simple_filter *		back();

    simple_predicate();
    virtual ~simple_predicate();

private:
    filter_list filters;
};

#include "simple_fwd_types.icc"

#endif
